// wengwengweng

use std::panic;
use std::panic::PanicInfo;

pub struct ErrorInfo {
	message: Option<String>,
	location: Option<ErrorLocation>,
}

pub struct ErrorLocation {
	file: String,
	line: u32,
}

pub fn set<F: 'static + Fn(ErrorInfo) + Send + Sync>(f: F) {

	panic::set_hook(Box::new(move |info: &PanicInfo| {

		let mut message = None;

		if let Some(s) = info.payload().downcast_ref::<&str>() {
			message = Some((*s).to_owned());
		} else if let Some(s) = info.payload().downcast_ref::<String>() {
			message = Some(s.clone());
		}

		let mut location = None;

		if let Some(loc) = info.location() {

			let file = loc.file();
			let line = loc.line();

			location = Some(ErrorLocation {
				file: file.to_owned(),
				line: line,
			});

		}

		f(ErrorInfo {
			message: message,
			location: location,
		});

	}));

}

